import json

import aiohttp
from aiocache import cached

from .exception import ValidateUIDError, UIDNotFounded
from .model import EnkaData
from .model.artifact import Artifact


class Enka:
    URL = "https://enka.shinshin.moe/u/{uid}/__data.json"
    LANG_URL = 'https://raw.githubusercontent.com/Dimbreath/GenshinData/master/TextMap/TextMap{lang}.json'
    AVATAR_URL = 'https://raw.githubusercontent.com/Dimbreath/GenshinData/master/ExcelBinOutput/AvatarExcelConfigData.json'
    USER_AGENT = "Mozilla/5.0"
    timeout = 5
    proxy = ''
    lang_data = {}
    avatar_data = {}
    lang = 'en'

    async def load_lang(self, lang='en'):
        async with aiohttp.ClientSession(headers={"User-Agent": self.USER_AGENT}) as client:
            if lang not in self.lang_data:
                resp = await client.get(self.LANG_URL.format(lang=lang.upper()), proxy=self.proxy)
                self.lang_data[lang] = json.loads(await resp.text())
            if not self.avatar_data:
                resp = await client.get(self.AVATAR_URL, proxy=self.proxy)
                for x in json.loads(await resp.text()):
                    self.avatar_data[x['id']] = x

    async def resolve_text_hash(self, text_hash, lang='en'):
        if lang not in self.lang_data:
            await self.load_lang(lang)
        if not isinstance(text_hash, str):
            text_hash = str(text_hash)
        if text_hash in self.lang_data[lang]:
            return self.lang_data[lang][text_hash]
        else:
            return ''

    @cached(ttl=600)
    async def fetch_user(self, uid: int) -> EnkaData:
        if not isinstance(uid, int):
            try:
                uid = int(uid)
            except ValueError:
                raise ValidateUIDError("Validate UID failed. Please check your UID.")
        if len(str(uid)) != 9 or (100000000 > uid > 999999999):
            raise ValidateUIDError("Validate UID failed. Please check your UID.")

        async with aiohttp.ClientSession(headers={"User-Agent": self.USER_AGENT},
                                         timeout=aiohttp.ClientTimeout(total=self.timeout)) as client:
            resp = await client.get(self.URL.format(uid=uid), proxy=self.proxy)

            if resp.status != 200:
                raise UIDNotFounded(f"UID {uid} not found.")

            data = await resp.json()

            if not data:
                raise UIDNotFounded(f"UID {uid} not found.")

        obj = EnkaData.parse_obj(data)
        if self.lang and self.lang in self.lang_data and self.lang_data[self.lang] and self.avatar_data:
            for character in obj.characters:
                for equip in character.equipList:
                    equip.flat.nameText = await self.resolve_text_hash(equip.flat.nameTextMapHash, self.lang)
                    if isinstance(equip, Artifact):
                        equip.flat.setNameText = await self.resolve_text_hash(equip.flat.setNameTextMapHash, self.lang)
                if character.id in self.avatar_data:
                    character.name = await self.resolve_text_hash(self.avatar_data[character.id]['nameTextMapHash'], self.lang)
        return obj



from enum import Enum
from typing import List, Dict, Union, Optional

from pydantic import BaseModel, Field, validator

from .artifact import Artifact
from .combats import CharacterCombat
from .weapon import Weapon


class CharacterPropertyType(int, Enum):
    UNKNOWN = -1
    XP = 1001
    ASCENSION = 1002
    LEVEL = 4001


class CharacterFriendshipLevel(BaseModel):
    level: int = Field(0, alias="expLevel")


class CharacterProperty(BaseModel):
    type: CharacterPropertyType
    ival: str = Field("", alias="ival")
    value: str = Field("", alias="val")

    @validator('type', pre=True)
    def check_type(cls, val: int):
        if val not in [1001, 1002, 4001]:
            val = -1

        return val

    class Config:
        use_enum_values = True


class CharacterInfo(BaseModel):
    name: Optional[str] = Field('')
    id: int = Field(0, alias="avatarId")
    friendship: CharacterFriendshipLevel = Field({}, alias="fetterInfo")
    properties: Dict[str, CharacterProperty] = Field({}, alias="propMap")
    # Artifacts
    equipList: List[Union[Artifact, Weapon]] = Field([], alias="equipList")
    combat: CharacterCombat = Field({}, alias="fightPropMap")
    skill_data: List[int] = Field([], alias="inherentProudSkillList")
    skill_id: int = Field(0, alias="skillDepotId")
    skillDepotId: int
    inherentProudSkillList: List[int]
    skill_level: Dict[int, int] = Field({}, alias="skillLevelMap")

    @property
    def artifacts(self):
        artifacts = []
        for _ in self.equipList:
            if isinstance(_, Artifact):
                artifacts.append(_)
        return artifacts

    @property
    def weapon(self):
        for _ in self.equipList:
            if isinstance(_, Weapon):
                return _



from typing import List,Optional

from pydantic import BaseModel


class ProfilePicture(BaseModel):
    avatarId: int


class ShowAvatar(BaseModel):
    avatarId: str
    level: int


class PlayerInfo(BaseModel):
    # Profile info
    finishAchievementNum: int
    level: int
    nameCardId: int
    nickname: str
    signature: Optional[str]
    worldLevel: int
    profilePicture: ProfilePicture
    # Avatars
    showAvatarInfoList: List[ShowAvatar]
    # Abyss floor
    towerFloorIndex: int
    towerLevelIndex: int



from typing import Dict, List, Optional

from pydantic import BaseModel, Field


class WeaponInfo(BaseModel):
    level: int
    affixMap: Dict[int, int]
    promoteLevel: int


class WeaponStats(BaseModel):
    appendPropId: str
    statValue: int


class WeaponFlat(BaseModel):
    icon: str
    itemType: str
    nameTextMapHash: str
    nameText: Optional[str] = Field('')
    rankLevel: int
    weaponStats: List[WeaponStats]


class Weapon(BaseModel):
    id: int = Field(0, alias="itemId")
    data: WeaponInfo = Field({}, alias="weapon")
    flat: WeaponFlat = Field({})



from typing import List, Optional

from pydantic import BaseModel, Field


class ArtifactProperty(BaseModel):
    mainPropId: str = Field('', alias='mainPropId')
    statValue: float = Field(0, alias='statValue')
    appendPropId: str = Field('', alias='appendPropId')

    @property
    def prop(self):
        if self.mainPropId:
            return self.mainPropId
        else:
            return self.appendPropId

    @property
    def value(self):
        return self.statValue


class ArtifactFlat(BaseModel):
    equipType: str
    icon: str
    itemType: str
    nameTextMapHash: str
    nameText: Optional[str] = Field('')
    rankLevel: int
    main_stat: ArtifactProperty = Field({}, alias="reliquaryMainstat")
    sub_stats: List[ArtifactProperty] = Field({}, alias="reliquarySubstats")
    setNameTextMapHash: str
    setNameText: Optional[str] = Field('')

    @property
    def icon_url(self):
        return f'https://enka.shinshin.moe/ui/{self.icon}.png'


class ArtifactInfo(BaseModel):
    level: int
    mainPropId: int
    appendPropIdList: List[int]


class Artifact(BaseModel):
    id: int = Field(0, alias="itemId")
    data: ArtifactInfo = Field({}, alias="reliquary")
    flat: ArtifactFlat
